% Matlab file for_ paper "Solving Heterogeneous Agent Models by Projection and Perturbation"
% Michael Reiter, IHS, September 2006
% Last update: June 2008
% Feel free to use, copy and modify at your own risk;
%   this program comes with NO WARRANTY WHATSOEVER
function [par,Kequ,D,G1,impact,abseig] = hetsolve(Kequ,parstart)
  global Params;

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % MODEL PARAMETERS:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % number of exogenous shocks:
  Params.nz = 2;
  % tax in steady state:
  Params.taustst = 0.0;
  % borrowing constraint:
  Params.kmin = 0;
  % Parameters of Pareto distribution, as a function_ of gamPareto:
  Params.minxPareto =  (1-1/Params.gamPareto);
  Params.CPareto =  Params.minxPareto^Params.gamPareto;  %to make sure that EY=1
  % parameters log-normal distribution:
  Params.sigY = 0.2;
  Params.muY = 0 - 0.5*Params.sigY^2;  %mean-adjustment for_ lognormal


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % DISCRETE APPROXIMATION OF INCOME PROCESS:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if(Params.iPareto)
    % truncate upper 1e-5 fraction from distribution
    trunc = 1e-5;
    p2 = [0.01;0.001;0.0001;1e-5];
    p = (1-sum(p2)-trunc)*ones(Params.ny-4,1)/(Params.ny-4);
    p = [p;p2];
    cp = cumsum(p);
    [Params.xY,Params.ymax] = discr_pareto(cp,Params.CPareto,Params.gamPareto);
    % adjust probabilities: for the distribution, later done  by adjDist!
    p = p/sum(p);
    Params.ymin = Params.minxPareto;
    Params.wY = p;
  else
    % lognormal distribution:
    [Params.xY,Params.wY] = qnwnorm(Params.ny,Params.muY,Params.sigY^2);
    Params.xY = exp(Params.xY);
    % upper bound support individual productivity distribution
    Params.ymax = max(Params.xY);
    Params.ymin = min(Params.xY);
  end
  Params.adjDist = 1;
  Params.adjDist = 1/disty(Params.ymax-1e-9,1);
  Params.ymean = momenty(1);
  EY = sum(Params.wY.*Params.xY);
  if(abs(EY-1)>1e-6)
    warning(sprintf('EY deviates from 1 by %e',1-EY));
  end
  assert(abs(EY-Params.ymean)<1e-8);
  
  
  % Upper limit capital:
  Params.kmax = 3*Params.ymax;


  % CHECK COMPATIBILITY OF R AND K
  % take into account that Y does not add up exactly to 1 because of truncation
  R = 1/Params.beta;
  Params.A = (R-1+Params.delta)/Params.alpha;
  KLratio = Params.ymean;  %ymean is average productivity
  assert(abs(R-1-netintr(KLratio,1))<1e-8);



  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % PREPARE THE INTEGRAL OF THE DISTRIBUTION FUNCTION_, F^(1)(xi) AS SCHUMAKER SPLINE
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % grid points for_ spline:
  n = 1000;
  ygrid = linspace(Params.ymin,Params.ymax,n)';
  IDistxi = zeros(n,1);
  for i=2:length(ygrid)
    % integrate the distribution function_ F(xi)
    IDistxi(i) = IDistxi(i-1) + quadl(@disty,ygrid(i-1),ygrid(i),[],[],1);
  end
  ff = find(IDistxi>0);
  n0 = ff(1)-1;  % drop initial points where distribution function is 0
  distyy = disty(ygrid,1);
  ff = find(distyy==1);
  n1 = ff(1);  %drop points where distribution function is already = 1
  % Schumaker spline:
  [Params.splineIF.x,Params.splineIF.y,Params.splineIF.dy] = ...
      schumaker_convex(ygrid(n0:n1),IDistxi(n0:n1),distyy(n0:n1));


  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % quadrature grid for_ wealth distribution:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if(Params.SmoothDev)
    Params.nStatesDistr = Params.nSpline;
  else
    Params.nStatesDistr = Params.ndstst;
  end
  % number knot points:
  ndk = Params.ndstst + 1;
  [Params.knotDistrK,Params.logshift] = makeknotd(Params.kmin,Params.kmax,ndk);





  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % SPLINE BASIS:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if(Params.SmoothDev)
    [Params.B,Params.Binv] = splinebasis(Params.knotDistrK,Params.nSpline);
  end


  
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % KNOT POINTS FOR_ SAVINGS POLYNOMIAL:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  xmin = 0.1*Params.ymin;
  xmax = Params.kmax*0.5; % beyond that point, linear extrapolation is fine!
  n1 = 50;
  x1 = linspace(xmin,0.2,n1+1)';
  x2 = logspaceshift(0.2,xmax,Params.nc-n1,Params.logshift)';
  Params.knotXi = [x1(1:end-1);x2];
  %  Params.knotXi = logspaceshift(xmin,xmax,Params.nc,Params.logshift)';
  Params.knotXi = Params.knotXi(2:end);

  if(nargout==0)
    return;
  end



  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % FIND STEADY STATE:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  if(nargin>=2)  %Kequ and starting values for_ parameters given
    Params.parstart = parstart;
  else  %have to solve for_ Kequ
    %STARTING VALUES FOR_ CONSUMPTION FUNCTION:
    par0 = [0.1;0.95*Params.knotXi];
    Params.parstart = par0;
    if(nargin==1)
      Kstart = Kequ;
    else
      if(Params.iPareto)
	Kstart = [1;4];
      else
	Kstart = [1;1.05];
      end
    end
    % COMPUTE STEADY STATE:
    opts = optimset('fzero');
    opts.TolX = 1e-10;
    Kequ = fzero(@ststresid,Kstart,opts);
  end
  [resid,par,D] = ststresid(Kequ);
  fprintf(1,'\nStSt done!!!!!\n');

  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % CHECK EULER RESIDUALS IN STEADY STATE:
  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  % par(1) is kink point of consumption function_:
  xx2 = linspace(par(1),xmax,10000)';
  R = 1+netintr(Kequ,1)-Params.taustst;
  wage = wagefunc(Kequ,1);
  Transf = Params.taustst*Kequ;
  resC = eulerres(par,par,R,wage,Transf,xx2);
  Params.eulerres = [max(abs(resC)) mean(abs(resC))];

  if(nargout<4)
    return;
  end

  % SOLVE LINEARIZED MODEL:
  ntotal = Params.nc+Params.nStatesDistr + Params.nz;
  Params.ix = 1:ntotal;
  Params.ixlag = ntotal+Params.ix;
  nn = Params.ixlag(end);
  Params.ieta = nn+1:nn+Params.nc;
  Params.ieps = nn+Params.nc+1:nn+Params.nc+Params.nz;
  [G1,impact,eu,abseig] = dosims(@equ,[zeros(Params.nStatesDistr,1);par;zeros(Params.nz,1)],Params.ix,Params.ixlag,Params.ieps,Params.ieta);
  if(~all(eu==1))  %signal failure:
    G1 = [];
    error(sprintf('ERROR: existence = %d; uniqueness = %d',eu(1),eu(2)));
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FUNCTION THAT HAS TO BE ZERO IN STEADY STATE:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [resid,par,D] = ststresid(K)
  global Params;
  R = 1+netintr(K,1)-Params.taustst;
  wage = wagefunc(K,1);
  Transf = Params.taustst*K;

  % GIVEN K, SOLVE FOR THE CONSUMPTION FUNCTION BY COLLOCATION:
  % high precision in solution of savings function_ is required to achieve
  %   converge in outer loop!
  [par,check] = broydn(@eulerres_stst,Params.parstart,1e-11,0,0,R,wage,Transf);
  if(check~=0)
    warning('broydn not converged');
  end
  Params.parstart = par;
  Pi = forwardmat(par,Transf,R,wage,Params.knotDistrK);
  D = invdistr(sparse(Pi));
  clear Pi;

  Params.Dstst = D;
  Params.parstst = par;

  K2 = expect_k(D);
  resid = K2 - K;
  fprintf(1,'K = %f, resid = %0.2e;  ',K-1, resid);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FUNCTION THAT DEFINES DISCRETE MODEL:
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% defines equations of stochastic model;
% Input:     X: all variables, current and lagged
% Outputs: residuals


function z = eulerres_stst(par,R,wage,Transf)
  z = eulerres(par,par,R,wage,Transf);


  

  


